#include <stdio.h>
#include <math.h>
#include <avs/flow.h> 
#include <avs/avs_data.h>
#include <avs/field.h>
#define PERFRAME 30

extern int svcorp2_();    /* European Stochastic in PVM */ 
extern int pvminit_();
extern int pvmdestroy_();

AVSinit_modules()
{
  int fin();

  AVSmodule_from_desc(fin);
}

static int reps1,nodes=8;

finance_init() 
{
  int who = 0;
       
  if (nodes>=32) {
    nodes = 32;
    reps1 = 5;
  }
  else if (nodes >= 16) {
    nodes = 16;
    reps1 = 4;
  }
  else if (nodes >= 8) {
    nodes = 8;
    reps1 = 3;
  }
  else if (nodes >= 4) {
    nodes = 4;
    reps1 = 2;
  }
  else if (nodes>=2) {
    nodes = 2;
    reps1 = 1;
  }
  else {
    nodes = 1;
    reps1 = 0;
  }
  fprintf(stderr,"\n Loading node(eu) programs ... \n");  
  pvminit_(&who,&nodes);

  return;
}

finance_destroy() 
{

  fprintf(stderr,"\n Destroy node(eu) programs ... \n");
  pvmdestroy_();

  return;
}

/*  The routine "fin" is the description routine.  */
fin()
{
  int fin_compute();     /* declare the compute function (below) */
  int finance_init();        /* declare the initialization function (below) */
  int finance_destroy(); /* declare the destroy function (below) */
  int in_port,out_port;  /* temporary to hold the port number */
  int parm;              /* temporary to hold the parm number */

  /* Set the module name and type */
  AVSset_module_name("European_pvm", MODULE_MAPPER);
  
  /* Create an input port for the required field input */
  in_port = AVScreate_input_port("Input Field",
                                 "field 1D uniform scalar float", REQUIRED);
  
  /* Create output ports for the result */
  out_port = AVScreate_output_port("Field Output", 
                                   "field 1D scalar uniform float");
  out_port = AVScreate_output_port("EUS_bias", "real");  
 
  /* Add parameter:  */
  parm = AVSadd_float_parameter("Psi", 1.0, 0.0, 10.0);
  parm = AVSadd_float_parameter("Rho", 0.5, -2.0, 2.0);
  parm = AVSadd_float_parameter("V_EUS", 1.0, 0.0, 5.0);
  parm = AVSadd_parameter("points/frame","integer", PERFRAME, 1, 100);

  /* interface to call pvm functions */
  AVSset_init_proc(finance_init);
  AVSset_destroy_proc(finance_destroy);

  /* Tell avs what subroutine to call to do the compute */
  AVSset_compute_proc(fin_compute);

}

/* extern int ASVCORP2_();*/
/*
 * fin_compute is the compute routine.  It is called whenever AVS wants to 
 * compute new model results.  The arguments are: the value of the input
 * field, the new output field (doubly indirected), the parameter 
 * values in the order same as declaration above.
 * Note the order is always inputs, outputs, parameters. 
 */

static float v_eus;
static int reps = 17;
static float xmu = 0.0;
static float sz,k,rate,t,tdiv,pdiv,call,psi,rho;

fin_compute(input, output, pbias, ppsi, prho, pv_eus, size)
AVSfield_float *input, **output;
float *ppsi, *prho, *pv_eus, **pbias;
int size;
{
  register int i;
  AVSfield_float *temp_field;

  int dims[1];
  float model_price;
  static float bias;
  float szdiv;
  float *dataptr = input->data;

  /* Allocate space for output */
  dims[0] = size;
  temp_field = (AVSfield_float *)AVSdata_alloc
    ("field 1D scalar uniform float",dims);

  if (temp_field == NULL) {
    AVSerror("Allocation of output field failed.");
    return(0);
  }

  if (*output) {  
    for (i=0; i<(size-1); i++) 
      temp_field->data[i] = (*output)->data[i+1];
    AVSdata_free("field",*output);
  }

  sz = *(dataptr++);  if (sz<0.0) sz = -sz;
  k = *(dataptr++); rate = *(dataptr++);
  t = *(dataptr++); tdiv = *(dataptr++); 
  pdiv = *(dataptr++); call = *dataptr;
  
  if (sz<0.0) sz = -sz;

  szdiv = sz - pdiv/exp(rate*tdiv);  
  svcorp2_(&szdiv,&k,&rate,&t,pv_eus,ppsi,
     &xmu,prho,&reps,&reps1,&model_price); 

  temp_field->data[size-1] = bias = model_price - call;
  fprintf(stderr," EUS bias=%8.5f",bias);
  fflush(stderr);

  *output = temp_field;
  *pbias = &bias; 

  /* When we're done, we return 1 to indicate success */
  return(1);
}



